<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Service;
use App\Models\ServiceCategory;
use Yajra\DataTables\Facades\DataTables;
use Illuminate\Database\QueryException;
use App\Models\ServiceLog;
use App\Models\User;
use App\Models\Stock;
use Carbon\Carbon;

class ServiceController extends Controller
{
    // ****************** service category 
    public function saveServiceCategory(Request $request)
    {
        $service = new ServiceCategory();
        $service->name = $request->name;
        $service->save();
        if ($service) {
            return response()->json(
                array(
                    'status' => true,
                    'data' => "Data added Successfully"
                ),
                200
            );
        } else {
            return response()->json(
                array(
                    'status' => false,
                    'data' => "Something went Wrong"
                ),
                200
            );
        }
    }
    public function updateServiceCategory(Request $request)
    {
        $service = ServiceCategory::where('id', $request->id)
            ->update(['name' => $request->name]);
        if ($service) {
            return response()->json(
                array(
                    'status' => true,
                    'data' => "Data updated Successfully"
                ),
                200
            );
        } else {
            return response()->json(
                array(
                    'status' => false,
                    'data' => "Something went Wrong"
                ),
                200
            );
        }
    }

    public function getServiceCategory(Request $request)
    {
        if ($request->ajax()) {
            $data = ServiceCategory::orderBy('id', 'DESC')->get();
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('name', function ($data) {
                    return   $data->name;
                })
                ->addColumn('action', function ($data) {
                    $actionBtn = '<a href="javascript:void(0);" data-id="' . $data->id . '" data-value="' . $data->name . '" id="getEditId" class="text-muted" data-toggle="modal" data-target="#exampleModalCenter2"> <span class="badge badge-primary"><i class="feather icon-edit-2"></i></span></a>
                     <a href="javascript:void(0);"  data-id="' . $data->id . '"  id="getDeleteId" data-toggle="modal" data-target="#delete_data" class="text-muted" > <span class="badge badge-success"><i class="feather icon-trash"></i></span></a>
                      
                                           ';
                    return $actionBtn;
                })


                ->escapeColumns([])

                ->make(true);
        }
    }


    public function deleteServiceCategory(Request $request)
    {
        try {
            $data = ServiceCategory::where('id', $request->id)->delete();
            return $data;
        } catch (QueryException $e) {
            return $e->getMessage();
        }
    }


    // ****************** service category  end


    // ****************** service  
    public function saveService(Request $request)
    {
        $service = new Service();
        $service->name = $request->name;
        $service->service_category_id = $request->category;
        $service->save();
        if ($service) {
            return response()->json(
                array(
                    'status' => true,
                    'data' => "Data added Successfully"
                ),
                200
            );
        } else {
            return response()->json(
                array(
                    'status' => false,
                    'data' => "Something went Wrong"
                ),
                200
            );
        }
    }


    public function getService(Request $request)
    {
        if ($request->ajax()) {
            $data = Service::orderBy('id', 'DESC')->get();
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('name', function ($data) {
                    return   $data->name;
                })
                ->addColumn('category', function ($data) {
                    return   $data->serviceCategory->name;
                })
                ->addColumn('action', function ($data) {
                    $actionBtn = '<a href="javascript:void(0);"   data-id="' . $data->id . '" data-value="' . $data->service_category_id . '" data-name="' . $data->name . '" id="getEditId" class="text-muted" data-toggle="modal" data-target="#exampleModalCenter2"> <span class="badge badge-primary"><i class="feather icon-edit-2"></i></span></a>
                     <a href="javascript:void(0);"  data-id="' . $data->id . '"  id="getDeleteId" data-toggle="modal" data-target="#delete_data" class="text-muted" > <span class="badge badge-success"><i class="feather icon-trash"></i></span></a>
                      
                                           ';
                    return $actionBtn;
                })


                ->escapeColumns([])

                ->make(true);
        }
    }


    public function deleteService(Request $request)
    {
        try {
            $data = Service::where('id', $request->id)->delete();
            return $data;
        } catch (QueryException $e) {
            return $e->getMessage();
        }
    }

    public function updateService(Request $request)
    {

        $service = Service::where('id', $request->id)
            ->update([
                'name' => $request->name,
                'service_category_id' => $request->category
            ]);
        if ($service) {
            return response()->json(
                array(
                    'status' => true,
                    'data' => "Data updated Successfully"
                ),
                200
            );
        } else {
            return response()->json(
                array(
                    'status' => false,
                    'data' => "Something went Wrong"
                ),
                200
            );
        }
    }
    // ****************** service   end

    public function saveServicelog(Request $request)
    {
        $cat = Service::whereIn('id', $request->services)->get('service_category_id');
        foreach ($cat as $c) {
            $sc[] = $c->service_category_id;
        }
        foreach ($request->services as $item) {
            foreach (Stock::where('service_id', $item)->get() as $s) {
                if ($s->quantity > 0) {
                    Stock::where('id', $s->id)->update(['quantity' => ($s->quantity - $request->quantity)]);
                }
            }
        }
        $log = new ServiceLog();
        $log->patient_id = $request->patient;
        $log->service_category_id = json_encode($sc);
        $log->service_id = json_encode($request->services);
        $log->volunteer_id = auth()->user()->id;
        $log->service_date = $request->date;
        $log->service_time = $request->time;
        $log->quantity = $request->quantity;
        $log->remarks = $request->remarks;
        $log->save();


        if ($log) {
            return response()->json(
                array(
                    'status' => true,
                    'data' => "Data added Successfully"
                ),
                200
            );
        } else {
            return response()->json(
                array(
                    'status' => false,
                    'data' => "Something went Wrong"
                ),
                200
            );
        }
    }
    public function updateServicelog(Request $request)
    {
        $cat = Service::whereIn('id', $request->services)->get('service_category_id');
        foreach ($cat as $c) {
            $sc[] = $c->service_category_id;
        }

        foreach ($request->services as $item) {
            foreach (Stock::where('service_id', $item)->get() as $s) {
                if ($s->quantity > 0) {
                    Stock::where('id', $s->id)->update(['quantity' => ($s->quantity - $request->quantity)]);
                }
            }
        }
        $service = ServiceLog::where('id', $request->id)
            ->update([
                'patient_id' => $request->patient,
                'service_category_id' => json_encode($sc),
                'service_id' => json_encode($request->services),
                'volunteer_id' => auth()->user()->id,
                'service_date' => $request->date,
                'service_time' => $request->time,
                'quantity' => $request->quantity,
                'remarks' => $request->remarks,
            ]);
        if ($service) {
            return response()->json(
                array(
                    'status' => true,
                    'data' => "Data updated Successfully"
                ),
                200
            );
        } else {
            return response()->json(
                array(
                    'status' => false,
                    'data' => "Something went Wrong"
                ),
                200
            );
        }
    }

    public function getServicelog(Request $request)
    {
        if ($request->ajax()) {
            $data = Servicelog::orderBy('id', 'DESC')->get();
            if ($request->filled('service')) {
                $data = $data->where('service_id', $request->service);
            }
            if ($request->filled('patient')) {
                $data = $data->where('patient_id', $request->patient);
            }
            if ($request->filled('user')) {
                $data = $data->where('volunteer_id', $request->user);
            }
            if ($request->filled('start_date') && $request->filled('end_date')) {
                $startdate = $request->start_date;
                $enddate = $request->end_date;

                $carbonStartDate = Carbon::parse($startdate);
                // Subtract one day to get yesterday's date
                $fromDate = $carbonStartDate->subDay();

                $carbonEndDate = Carbon::parse($enddate);
                // Subtract one day to get yesterday's date
                $toDate = $carbonEndDate->subDay();


                $data = $data->whereBetween('service_date', [$fromDate, $toDate]);
            }
            return Datatables::of($data)
                ->addIndexColumn()

                ->addColumn('patient', function ($data) {
                    return   '<a href="' . route('patient-view', ['id' => $data->patient->id]) . '">' . $data->patient->name . '</a>';
                })
                ->addColumn('volunteer', function ($data) {
                    return   User::where('id', $data->volunteer_id)->pluck('name')->first();;
                })
                ->addColumn('category', function ($data) {
                    $html = "";
                    foreach (json_decode($data->service_category_id) as $item) {
                        $html .= '<p>' . ServiceCategory::where('id', $item)->pluck('name')->first() . '</p><br>';
                    }

                    return $html;
                })
                ->addColumn('service', function ($data) {
                    $html = "";
                    foreach (json_decode($data->service_id) as $item) {
                        $html .= '<p>' . Service::where('id', $item)->pluck('name')->first() . '</p><br>';
                    }

                    return $html;
                })
                ->addColumn('date', function ($data) {
                    return   $data->service_date;
                })
                ->addColumn('time', function ($data) {
                    return   $data->service_time;
                })
                ->addColumn('quantity', function ($data) {
                    return   $data->quantity;
                })
                ->addColumn('remarks', function ($data) {
                    return   $data->remarks;
                })
                ->addColumn('action', function ($data) {
                    $actionBtn = '
                    <a href="' . route('edit-service-log', ['id' => $data->id]) . '" > <span class="badge badge-primary"><i class="feather icon-edit-2"></i></span></a>
                     <a href="javascript:void(0);"  data-id="' . $data->id . '"  id="getDeleteId" data-toggle="modal" data-target="#delete_data" class="text-muted" > <span class="badge badge-success"><i class="feather icon-trash"></i></span></a>
                      
                                           ';
                    return $actionBtn;
                })


                ->escapeColumns([])

                ->make(true);
        }
    }
    public function getPatientServicelog(Request $request)
    {
        if ($request->ajax()) {
            $data = Servicelog::where('patient_id', $request->id)->orderBy('id', 'DESC')->get();
            return Datatables::of($data)
                ->addIndexColumn()

                ->addColumn('patient', function ($data) {
                    return   $data->patient->name;
                })
                ->addColumn('volunteer', function ($data) {
                    return   User::where('id', $data->volunteer_id)->pluck('name')->first();;
                })
                ->addColumn('service', function ($data) {
                    return   $data->service->name;
                })
                ->addColumn('date', function ($data) {
                    return   $data->service_date;
                })
                ->addColumn('time', function ($data) {
                    return   $data->service_time;
                })
                ->addColumn('quantity', function ($data) {
                    return   $data->quantity;
                })
                ->addColumn('remarks', function ($data) {
                    return   $data->remarks;
                })
                ->addColumn('action', function ($data) {
                    $actionBtn = '
                     <a href="javascript:void(0);"  data-id="' . $data->id . '"  id="getDeleteId" data-toggle="modal" data-target="#delete_data" class="text-muted" > <span class="badge badge-success"><i class="feather icon-trash"></i></span></a>
                      
                                           ';
                    return $actionBtn;
                })


                ->escapeColumns([])

                ->make(true);
        }
    }

    public function editServicelog(Request $request)
    {
        $data = ServiceLog::where('id', $request->id)->first();
        return view('admin.service-log-edit', ['data' => $data]);
    }
    public function deleteServicelog(Request $request)
    {
        try {
            $data = Servicelog::where('id', $request->id)->delete();
            return $data;
        } catch (QueryException $e) {
            return $e->getMessage();
        }
    }


    // ****************** Stock
    public function saveStock(Request $request)
    {
        $service = new Stock();
        $service->service_id = $request->category;
        $service->name = $request->name;
        $service->quantity = $request->quantity;
        $service->save();
        if ($service) {
            return response()->json(
                array(
                    'status' => true,
                    'data' => "Data added Successfully"
                ),
                200
            );
        } else {
            return response()->json(
                array(
                    'status' => false,
                    'data' => "Something went Wrong"
                ),
                200
            );
        }
    }


    public function getStock(Request $request)
    {
        if ($request->ajax()) {
            $data = Stock::orderBy('id', 'DESC')->get();
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('name', function ($data) {
                    return   $data->name;
                })
                ->addColumn('service', function ($data) {
                    return   Service::where('id', $data->service_id)->pluck('name')->first();
                })
                ->addColumn('quantity', function ($data) {
                    return   $data->quantity;
                })
                ->addColumn('action', function ($data) {
                    $actionBtn = '<a href="javascript:void(0);"   data-id="' . $data->id . '" data-value="' . $data->service_id . '" data-name="' . $data->name . '" data-quantity="' . $data->quantity . '"  id="getEditId" class="text-muted" data-toggle="modal" data-target="#exampleModalCenter2"> <span class="badge badge-primary"><i class="feather icon-edit-2"></i></span></a>
                     <a href="javascript:void(0);"  data-id="' . $data->id . '"  id="getDeleteId" data-toggle="modal" data-target="#delete_data" class="text-muted" > <span class="badge badge-success"><i class="feather icon-trash"></i></span></a>
                      
                                           ';
                    return $actionBtn;
                })


                ->escapeColumns([])

                ->make(true);
        }
    }


    public function deleteStock(Request $request)
    {
        try {
            $data = Stock::where('id', $request->id)->delete();
            return $data;
        } catch (QueryException $e) {
            return $e->getMessage();
        }
    }

    public function updateStock(Request $request)
    {

        $service = Stock::where('id', $request->id)
            ->update([
                'service_id' => $request->category,
                'name' => $request->name,
                'quantity' => $request->quantity
            ]);
        if ($service) {
            return response()->json(
                array(
                    'status' => true,
                    'data' => "Data updated Successfully"
                ),
                200
            );
        } else {
            return response()->json(
                array(
                    'status' => false,
                    'data' => "Something went Wrong"
                ),
                200
            );
        }
    }
    // ****************** Stock end

    public function loadService(Request $request)
    {

        $service = Service::where('service_category_id', $request->id)->get();
        // print_r($service);
        // die();
        // $html = '';
        // foreach ($service as $item) {
        //     $html = '<option value="{{$item->id}}">{{$item->service_name}}</option>';
        // }
        return response()->json($service);
    }
}
